/* This file is part of kmobiletools.
   See COPYING file for licensing information.

   (C) 2005 Marco Gulino <marco.gulino@gmail.com>
   (C) 2005 Stefan Bogner <bochi@kmobiletools.org>

*/

#include "gammu_jobs.h"
#include "qphoneerror.h"

#include "qregexp.h"
#include <unistd.h>
#include <qstringlist.h>
#include <qdatetime.h>
#include "kmobiletoolsgammu_engine.h"
#include <kdebug.h>

FetchPhoneInfos::FetchPhoneInfos(Device *dev, kmobiletoolsGammu_engine* parent, const char* name) : kmobiletoolsGammuJob(dev, parent, name){}

void FetchPhoneInfos::run()
{
    if( p_device->phone ) {

        p_device->phone->GetModel(&p_device->s);
        s_model = p_device->s.Phone.Data.ModelInfo->model;

        p_device->phone->GetManufacturer(&p_device->s);
        s_manufacturer = p_device->s.Phone.Data.Manufacturer;

        p_device->phone->GetFirmware(&p_device->s);
        s_firmware = p_device->s.Phone.Data.Version;

        p_device->phone->GetIMEI(&p_device->s);
        s_imei = p_device->s.Phone.Data.IMEI;
    }
}

PollStatus::PollStatus(Device *dev, kmobiletoolsGammu_engine* parent, const char* name) : kmobiletoolsGammuJob(dev, parent, name){}

void PollStatus::run()
{
    if ( p_device->phone ) {

        GSM_SignalQuality net;
        GSM_BatteryCharge bat;
        GSM_NetworkInfo info;
        GSM_SMSMemoryStatus sms;
        GSM_DisplayFeatures features;
        int i;

        error = p_device->phone->GetBatteryCharge( &p_device->s, &bat );
        kdDebug() << "[ !!! ] GetBatteryCharge: " << err.getErrorMsg(error) << endl;

        if ( error == ERR_NONE ) s_battery = bat.BatteryPercent;

        error = p_device->phone->GetNetworkInfo( &p_device->s, &info );
        kdDebug() << "[ !!! ] GetNetworkInfo: " << err.getErrorMsg(error) << endl;

        if( error == ERR_NONE )
        {
            if( !info.NetworkName[ 1 ] ) s_network = "";
            else s_network = QString::fromUtf8( (const char*) DecodeUnicodeConsole(&info.NetworkName[ 0 ]));
        }

        error = p_device->phone->GetSignalQuality( &p_device->s, &net );
        kdDebug() << "[ !!! ] GetSignalQuality: " << err.getErrorMsg(error) << endl;

        if( error == ERR_NONE ) s_signal = net.SignalPercent;

        error = p_device->phone->GetSMSStatus( &p_device->s, &sms );
        kdDebug() << "[ !!! ] GetSMSStatus: " << err.getErrorMsg(error) << endl;

        if( error == ERR_NONE )
        {
            s_unreadSMS = sms.SIMUnRead + sms.PhoneUnRead;
            s_totalSMS = sms.SIMUsed + sms.PhoneUsed;
        }
        kdDebug() << "[ !!! ] SMS " << s_unreadSMS << "unread, " << s_totalSMS << " total." << endl;

        /* GetDisplayStatus is the only way I found to figure out whether or not the phone
            is ringing. Unfortunately it is unsupported by my phone, so I can't test it... */

        error = p_device->phone->GetDisplayStatus( &p_device->s, &features );
        kdDebug() << "[ !!! ] GetDisplayStatus: " << err.getErrorMsg(error) << endl;
	
	s_ringing = false;

        if( error == ERR_NONE )
        {
            for ( i=0;i<features.Number;i++ ) {

                switch( features.Feature[i] ) {
                    case GSM_CallActive:
                        s_ringing = true;
                    break;
                    case GSM_UnreadSMS:
                        // TODO: Implement me!
                    break;
                    case GSM_VoiceCall:
                        // TODO: Implement me!
                    break;
                    case GSM_FaxCall:
                        // TODO: Implement me!
                    break;
                    case GSM_DataCall:
                        // TODO: Implement me!
                    break;
                    case GSM_KeypadLocked:
                        // TODO: Implement me!
                    break;
                    case GSM_SMSMemoryFull:
                        // TODO: Implement me!
                    ;
                }
            }
        }
    }
}

