/* This file is part of kmobiletools.
   See COPYING file for licensing information.

   (C) 2005 Marco Gulino <marco.gulino@gmail.com>
   (C) 2005 Stefan Bogner <bochi@kmobiletools.org>

*/

#ifndef KMOBILETOOLSKAMMU_ENGINE_H
#define KMOBILETOOLSKAMMU_ENGINE_H

#include "kmobiletoolsengine.h"
#include "qphoneerror.h"

extern "C" {
    #include <gammu/gammu.h>
}

#undef bool

#include "device.h"
#include <qptrlist.h>
#include <kabc/addressee.h>
#include <qptrvector.h>

using namespace ThreadWeaver;
class kmobiletoolsGammu_engine : public kmobiletoolsEngine
{
    Q_OBJECT

public:
    kmobiletoolsGammu_engine(QObject *parent = 0, const char *name = 0);

    ~kmobiletoolsGammu_engine();

public:
    Device* dev;
    QString getPhoneModel();
    bool connectEmitted;
    bool disconnectEmitted;
    int availPbSlots();
    
public slots:
    void dial(DialActions, const QString & =QString::null ) {}
    void pollPhoneStatus();
    void processSlot(Job* );
    void getPhoneInfos();
    void getSMSFolders();
    void initPhone();
    void retrieveAddressBook();
    void retrieveSMSList();
    QStringList encodings() { return QStringList(QString::null); }
    void slotAddAddressee(QValueList<KABC::Addressee>*);
    void slotDelAddressee(QValueList<KABC::Addressee>*);
    int getFreeEntry( GSM_MemoryType );
    void slotEditAddressee(KABC::Addressee* oldAddressee, KABC::Addressee* newAddressee);
    void setDevice ( const QString &deviceName){};
    void closeDevice() { delete dev;}
    const QStringList  probeDevice ( int probetype, const QString &deviceName, const QString &param1=QString::null, const QString &param2=QString::null){};
    virtual void probePhone(){};
    virtual void slotDelSMS(SMS*){};
    virtual void slotStoreSMS(const QString&, const QString&){};
    virtual void slotSendSMS(const QString&, const QString&){};
    virtual DeviceInfos probeDevice(ThreadWeaver::Job *job, bool, const QString&, const QStringList&)const { return DeviceInfos(); };
    virtual void slotStoreSMS(SMS*){};
    virtual void slotSendSMS(SMS*){};
    virtual bool pdu(){};
    virtual void slotSendStoredSMS(SMS*){};
    virtual void fetchCalendar(){};      
   
 
private:
    QValueList<GSM_MemoryEntry> freeMemEntries;

private slots:
    void devConnected();

};

class kmobiletoolsGammu_engineFactory : public KLibFactory
{
   Q_OBJECT

public:
    kmobiletoolsGammu_engineFactory();
    virtual ~kmobiletoolsGammu_engineFactory();
    virtual kmobiletoolsGammu_engine* createObject( QObject *parent, const char *name, const char *classname, const QStringList &args );


private:

};



#endif
