/***************************************************************************
   Copyright (C) 2006
   by Marco Gulino <marco@kmobiletools.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the
   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 ***************************************************************************/
#include "serialdevice.h"
#include <qmutex.h>
#include <kdebug.h>
#include <qtimer.h>
#include <qstring.h>
#include <qregexp.h>
#include <kapplication.h>
#include <qdir.h>
#include <qfile.h>
#include <kuser.h>
#include <qserial.h>
#include <weaver.h>

#include <iostream>

#include "engineslist.h"
#include "kmobiletools_devices.h"

using namespace std;
using namespace KMobileTools;

//TODO: Remove this define
// Define from at_jobs.cpp
#define CMDERROR (d->buffer==QString::null || d->buffer=="ERROR")

class KMobileTools::SerialManagerPrivate {
    public:
        SerialManagerPrivate() : /*lockfilecreated(-1), modem(-1), locked(false), connectEmitted(false), disconnectEmitted(false), timeouts(0), */
            b_connected(false), serial(0L), m_baudrate(QSerial::BAUD_57600)
        {
            mutex=new QMutex(true);
        }
        ~SerialManagerPrivate() { delete mutex; }

        bool b_connected;
//         QStringList commandQueueStack;
        QString buffer;
        QMutex *mutex;
        QSerial *serial;
        QString s_devicePath;
        QStringList deviceInitStrings;
        QSerial::Baud m_baudrate;
};

void SerialManager::lockMutex() { d->mutex->lock(); }
void SerialManager::unlockMutex() { d->mutex->unlock(); }


//TODO: Remove this function
// Function from at_jobs.cpp
QString parseInfo( QString buffer )
{
    QString tmp = buffer.section("OK\r\n",0,0).remove( '\r' ).remove( '\n' );
    int i = tmp.find( ':' );
    if ( i>0 && i<=6 && tmp.at(0)=='+' )
        tmp = tmp.section( ":",1 );
    tmp = tmp.stripWhiteSpace();
    if ( tmp.at(0)=='"' && tmp.at(tmp.length()-1)=='"' )
        tmp = tmp.mid( 1, tmp.length()-2 );
    return tmp;
}

SerialManager::SerialManager(QObject * parent, const char* name, const QString &devicePath, const QStringList &initStrings)
    : QObject(parent, name)
{
    d=new SerialManagerPrivate;
    if(devicePath.length() && QFile::exists( devicePath ) ) d->s_devicePath=devicePath;
    if(initStrings.count() ) d->deviceInitStrings=initStrings;
}

bool SerialManager::isConnected() { return d->b_connected; }
SerialManager::~SerialManager()
{
    close();
    delete d;
}

void SerialManager::setDevicePath(const QString &path)
{
    if(d->serial)
    {
        if(d->serial->isOpen())
        {
            close();
            setDevicePath(path);
            open(0);
        } else
        {
            d->serial->setName(path);
        }
    } else d->s_devicePath=path;
}

bool SerialManager::open(ThreadWeaver::Job *job)
{
    d->serial=new QSerial(d->s_devicePath);
    d->serial->setBaud( d->m_baudrate );
    d->serial->setStopBits( QSerial::STOP_BITS_1 );
    d->serial->setDatabits( QSerial::DATABITS_8 );
    d->serial->setFlowControl( QSerial::FLOW_CONTROL_HARD );
    d->serial->setParity( QSerial::PARITY_NONE );

    bool isOpen= d->serial->open(IO_ReadWrite/* | O_NOCTTY | O_NONBLOCK */);
    if(!isOpen) return false;
    connect(d->serial, SIGNAL(gotData()), this, SLOT(gotData()));
    d->serial->reset();
    sendATCommand(job, "ATZ\r");
//     sendATCommand(job, "AT\r", 100);

    for ( QStringList::Iterator it=d->deviceInitStrings.begin(); it!=d->deviceInitStrings.end(); ++it)
        if((*it).stripWhiteSpace().length()>1) sendATCommand(job, *it + "\r");
    emit connected();
    d->b_connected=true;
    return true;
}

#include "serialdevice.moc"
void SerialManager::close()
{
    if( !d->serial || !d->serial->isOpen() ) return;
    d->serial->close();
    delete d->serial;
    d->serial=0;
    d->b_connected=false;
    emit disconnected();
}

QString SerialManager::sendATCommand(ThreadWeaver::Job *job, const QString &cmd, uint timeout)
{
    QTime timer;
//     kdDebug() << "Mutex is locked::" << d->mutex->locked() << endl;
    QMutexLocker mlocker(d->mutex);
    if(!cmd.length() || ! d->serial || !d->serial->isOpen()) return QString::null;
//     QString classcmd=cmd.section( QRegExp("^AT"), 1,1,QString::SectionCaseInsensitiveSeps);
//     classcmd=classcmd.left(classcmd.find(QRegExp("[^\\w]"), 1) );
//     d->commandQueueStack+=classcmd;
//     kdDebug() << "QueueStack: " << d->commandQueueStack << endl;
//     timeout=(timeout * 1000)+1;

    d->buffer=QString::null;
    long err=d->serial->writeBlock(cmd.latin1(), cmd.length());
    if(err==-1)
    {
        kdDebug() << "Write error\n";
//         d->commandQueueStack.remove(classcmd);
        return QString::null;
    }
    d->serial->flush();
    std::cout << ">>>" << QString(cmd).replace("\r","\n").replace("\n\n", "\n") << endl;
    std::cout << "<<<";
    QRegExp exitExp("(OK|ERROR)(\\n|\\r)");
    timer.start();
    while ( d->serial && !EnginesList::instance()->closing()
            && ( (timeout==0) || ((timer.elapsed() <timeout) && !d->buffer.contains(exitExp) ) )) /// @TODO fix this loop to have something like timeout in msecs.
    {
        if( timer.elapsed() >=3000 && (timer.elapsed()%3000 == 0)) {d->serial->writeBlock("AT\r", 3);
            d->serial->flush();
            kdDebug() << "****************** WARNING!!!! Sending AT\\r to unblock the phone.\n";
            kdDebug() << "****************** this can be a bug of the phone, or of kmobiletools.\n";
            kdDebug() << "****************** please report to marco AT kmobiletools.org: AT command=" << cmd << endl;
        }
        if(job)
            job->thread()->msleep( 1 );
        else
            usleep(1000);
//             usleep(100000);
    }
    std::cout << endl;
    if(timer.elapsed() >=timeout &&
       !cmd.contains( QRegExp("AT\\+(CMGW|CMGS)=(\")*[\\d]+(\")*") ) // Some commands just NEED to exit with a timeout, don't generate a warning for them..
      )
    {
        kdDebug() << "****************** WARNING!!!! Phone seems to be locked.\n";
        kdDebug() << "****************** this can be a bug of the phone, or of kmobiletools.\n";
        kdDebug() << "****************** please report to marco AT kmobiletools.org: AT command=" << cmd << endl;
    }
    /// @TODO handle also partial errors
    exitExp.setPattern( "ERROR(\\n|\\r)");
    if( d->buffer.contains( exitExp ) ) d->buffer="ERROR";
//     d->commandQueueStack.remove(classcmd);
    return d->buffer;
}

KMobileTools::QSerial *SerialManager::qserial()
{
    return d->serial;
}

void SerialManager::gotData()
{
#define MAXBUFSIZE 32
    char buffer[MAXBUFSIZE];
    memset(buffer, 0, MAXBUFSIZE);
    int readdata=d->serial->readBlock(buffer, MAXBUFSIZE-1);
    if(readdata>0)
    {
        d->buffer+=buffer;
        std::cout << QString(buffer).replace("\r", "\n").replace("\n\n", "\n");
    }
}




/*!
    \fn SerialManager::speed(int value)
 */
 /// @TODO have to move this somewhere...
void SerialManager::setSpeed(int value)
{
    switch( value ){
        case 0:
            d->m_baudrate=QSerial::BAUD_9600;
            break;
        case 1:
            d->m_baudrate= QSerial::BAUD_38400;
            break;
        case 2:
            d->m_baudrate= QSerial::BAUD_57600;
            break;
        case 3:
            d->m_baudrate= QSerial::BAUD_115200;
            break;
        case 4:
            d->m_baudrate= QSerial::BAUD_230400;
            break;
        default:
            d->m_baudrate= QSerial::BAUD_57600;
            break;
    }
}

QString SerialManager::decodePDU( QString text )
{
    QString decoded;
    for ( uint i=0; i<text.length(); i += 2 )
    {
        decoded.append( QChar( text.mid( i, 2 ).toInt( 0,16 ) ) );
    }
    return decoded;
}
