/* This file is part of the KDE project
   Copyright (c) 2004 Kevin Ottens <ervin ipsquad net>
   Rearranged by Marco Gulino <marco@kmobiletools.org> for the mobile protocol

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/
#include "kio_p2kprotocol.h"

#include <qcstring.h>
#include <qsocket.h>
#include <qdatetime.h>
#include <qbitarray.h>

#include <stdlib.h>
#include <math.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#include <kapplication.h>
#include <kdebug.h>
#include <kmessagebox.h>
#include <kinstance.h>
#include <kglobal.h>
#include <kstandarddirs.h>
#include <klocale.h>
#include <kurl.h>
#include <ksock.h>
#include "kmobiletools_devices.h"
#include <qfile.h>
#include "p2kwrapper.h"
#include <unistd.h>
#include <dcopclient.h>


using namespace KIO;

kio_p2kProtocol::kio_p2kProtocol(const QCString &pool_socket, const QCString &app_socket)
    : SlaveBase("kio_p2k", pool_socket, app_socket)
{
    kdDebug() << "kio_p2kProtocol::kio_p2kProtocol\n";
    kmt_stub=0;
    p_dcop=new DCOPClient();
    p_dcop->attach();
}


kio_p2kProtocol::~kio_p2kProtocol()
{
    if ( kmt_stub ) kmt_stub->resumeDevice();

    kdDebug() << "kio_p2kProtocol::~kio_p2kProtocol\n";
}

bool kio_p2kProtocol::setHostConfig(const QString &hostname)
{
    kdDebug() << "kio_p2kProtocol::setHostConfig(" << hostname << ") with s_host==" << s_host << endl;
    if ( hostname== s_host ) return true;
    kdDebug() << "Trying to find config values for device " << hostname << endl;
//             kmt_stub->currentDeviceName();
    KMobileTools::DevicesConfig* Config=KMobileTools::DevicesConfig::prefs(hostname);
    QString atpath=Config->lastpath();
    kmt_stub=new DeviceIFace_stub( p_dcop, QCString("kmobiletools"), hostname.local8Bit() );
    kdDebug() << "Found values: " << atpath << "; " << Config->rawdevicevendor() << "; " << Config->rawdevicename() << endl;
    if ( ! atpath.length() || ! Config->rawdevicevendor().length() || ! Config->rawdevicename().length() ) return false;
    kdDebug() << "Trying to switch off kmobiletools...\n";
    if ( kmt_stub && ! kmt_stub->isConnected() ) kmt_stub->resumeDevice();
    kmt_stub->stopDevice();
    kdDebug() << "Device : " << kmt_stub->deviceModel() << endl;
    while ( kmt_stub->isConnected() ) { kdDebug() << "...waiting....\n"; sleep (1 ); }
    kdDebug() << "Success!!\n";

    if( m_impl.wrapper()->findDevice( atpath, Config->rawdevicevendor(), Config->rawdevicename() ) )
    {

        s_host=hostname;
        return true;
    }
    kmt_stub->resumeDevice();
    return false;
}

void kio_p2kProtocol::listDir(const KURL &url)
{
    kdDebug() << "P2KProtocol::listDir: " << url << endl;
    if (! setHostConfig( url.host() ) )return;

    if ( url.path().length() <= 1 )
    {
        listRoot(url);
        return;
    }


    KIO::UDSEntry entry;

    KIO::UDSEntryList mobile_entries;
    bool ok = m_impl.listDirectory(url,mobile_entries);

    if (!ok)
    {
        error( m_impl.lastErrorCode(), m_impl.lastErrorMessage() );
        return;
    }

    totalSize(mobile_entries.count()+1);

    m_impl.createTopLevelEntry(entry);
    listEntry(entry, false);

    KIO::UDSEntryListIterator it = mobile_entries.begin();
    KIO::UDSEntryListIterator end = mobile_entries.end();

    for(; it!=end; ++it)
    {
        listEntry(*it, false);
    }

    entry.clear();
    listEntry(entry, true);

    finished();
}

void kio_p2kProtocol::stat(const KURL &url)
{
    kdDebug() << "**************** P2KProtocol::stat: " << url << endl;
    if (! setHostConfig( url.host() ) )return;

//     listDir(url);
//     finished();
//     return;
    KIO::UDSEntry entry;
    if ( url.path().isEmpty() || url.path() == "/" )
    {
        m_impl.createTopLevelEntry( entry );
        statEntry( entry );
        finished();
        return;
    }
    bool ok=m_impl.statEntry( url, entry );
    if (!ok)
    {
        m_impl.createTopLevelEntry(entry);
        statEntry(entry);
        finished();
        return;
    }
    statEntry( entry );

    finished();
    return;
/*    QString path = url.path();
    if ( true || path.isEmpty() || path == "/" )
    {
                // The root is "virtual" - it's not a single physical directory
        KIO::UDSEntry entry;
        m_impl.createTopLevelEntry( entry );
        statEntry( entry );
        finished();
        return;
    }
*/
}

void kio_p2kProtocol::listRoot(const KURL &url)
{
    kdDebug() << "P2KProtocol::listRoot()\n";

    if (! setHostConfig( url.host() ) )return;

    KIO::UDSEntry entry;

    KIO::UDSEntryList mobile_entries;
    bool ok = m_impl.listRoot(url,mobile_entries);

    if (!ok)
    {
        error( m_impl.lastErrorCode(), m_impl.lastErrorMessage() );
        return;
    }

    totalSize(mobile_entries.count()+1);

    m_impl.createTopLevelEntry(entry);
    listEntry(entry, false);

    KIO::UDSEntryListIterator it = mobile_entries.begin();
    KIO::UDSEntryListIterator end = mobile_entries.end();

    for(; it!=end; ++it)
    {
        listEntry(*it, false);
    }

    entry.clear();
    listEntry(entry, true);

    finished();
}

extern "C"
{
    int kdemain(int argc, char **argv)
    {
        KInstance instance( "kio_p2k" );
        
        kdDebug(7101) << "*** Starting kio_p2k " << endl;
        
        if (argc != 4) {
            kdDebug(7101) << "Usage: kio_p2k  protocol domain-socket1 domain-socket2" << endl;
            exit(-1);
        }
        
        kio_p2kProtocol slave(argv[2], argv[3]);
        slave.dispatchLoop();
        
        kdDebug(7101) << "*** kio_p2k Done" << endl;
        return 0;
    }
}



/*!
    \fn kio_p2kProtocol::get(const KURL &url)
 */
void kio_p2kProtocol::get(const KURL &url)
{
    if (! setHostConfig( url.host() ) )return;

    kdDebug() << "***************** kio_p2k getFile: " <<  url << endl;
    int size=m_impl.getFile(url, 0);
    char* buffer=new char[size];
    m_impl.getFile(url,buffer);
    kdDebug() << "**************************GetFile processed; size returned: " << size <<"; raw data:" << buffer << endl;
    if(size<=0)
    {
        delete [] buffer;
        finished();
        return;
    }
    QByteArray m_data;
    m_data.setRawData( buffer,size);
    data( m_data );
    processedSize( m_data.size() );

    m_data.resetRawData( buffer, size );
    data(QByteArray());
    delete [] buffer;
    finished();
}


/*!
    \fn kio_p2kProtocol::put(const KURL &url, int permissions, bool overwrite, bool resume)
 */
void kio_p2kProtocol::put(const KURL &url, int permissions, bool overwrite, bool resume)
{
    if (! setHostConfig( url.host() ) )return;

    kdDebug() << "URL: " << url << "; permissions: " << permissions << "; overwrite: " << overwrite << "; resume: " << resume << endl;
    QByteArray buffer;
    dataReq();
    int result= readData( buffer );
    kdDebug() << "SendData result code: " << m_impl.wrapper()->putFile( url.path(), buffer.data(), buffer.size() );
    finished();
}


/*!
    \fn kio_p2kProtocol::del (const KURL &url, bool isfile)
 */
void kio_p2kProtocol::del (const KURL &url, bool isfile)
{
    if (! setHostConfig( url.host() ) )return;

    kdDebug() << "Deleting " << url << "; is file? " << isfile << "; result: " << m_impl.wrapper()->deleteFile( url.path() ) << endl;
    finished();
}
