/***************************************************************************
   Copyright (C) 2006
   by Marco Gulino <marco@kmobiletools.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the
   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 ***************************************************************************/
#include "p2kwrapper.h"
#include <kdebug.h>

#include <iostream>
#include <unistd.h>

using namespace std;

P2KWrapper::P2KWrapper(int vendor, int at_prodID, int p2k_prodID, QString acmDevice, QObject *parent, const char *name)
    : QObject(parent, name)
{
    i_vendor=vendor;
    i_at_prodID=at_prodID;
    i_p2k_prodID=p2k_prodID;
    b_connected=FALSE;
    s_acmDevice=acmDevice;

    p2k_init();
    p2k_setACMdevice( acmDevice.latin1() );
    p2k_setATconfig( vendor, at_prodID );
    p2k_setP2Kconfig( vendor, p2k_prodID );
    kdDebug() << "Initialized p2k lib\n";
    n_files=0;
}

P2KWrapper::P2KWrapper(QObject *parent, const char *name)
    : QObject(parent, name)
{
    b_connected=false;
    p2k_init();

    kdDebug() << "Initialized p2k lib\n";
    n_files=0;
}

bool P2KWrapper::findDevice(const QString &acmDevice, const QString &rawVendor, const QString &rawProduct)
{
    p2k_init();
    kdDebug() << "P2KWrapper::findDevice(" << acmDevice << "," << rawVendor << "," << rawProduct << ")\n";
    p2k_setACMdevice( acmDevice.latin1() );
    p2k_devInfo* devList=p2k_getDevList();
    int i=0;
    while( devList[i].vendor >=0 ){
        kdDebug() << "Current device infos: " << QString::number(devList[i].vendor,16)
                << "::" << QString::number(devList[i].product,16) << endl;
        if ( ! (devList[i].vendor==0x22b8) )
        {
            kdDebug() << "Vendor not found\n";
            i++; continue;
        }
        kdDebug() << "Found phone from vendor " << rawVendor << endl;
        QString model=QStringList::split( "MODEL=", rawProduct ).last();
        kdDebug() << "Phone model from p2k:" << devList[i].productStr << ";\n";
        if ( QString( devList[i].productStr ).contains( model ) )
        {
            p2k_setACMdevice( acmDevice.latin1() );
            if ( devList[i].product%2 )
            {
                kdDebug() << "Setting AT to " << QString::number(devList[i].product+1,16) << " and p2k to "
                        << QString::number(devList[i].product,16) << endl;
                p2k_setATconfig( devList[i].vendor, devList[i].product+1 );
                p2k_setP2Kconfig( devList[i].vendor, devList[i].product );
            } else
            {
                kdDebug() << "Setting AT to " << QString::number(devList[i].product,16) << " and p2k to "
                        << QString::number(devList[i].product-1,16) << endl;
                p2k_setATconfig( devList[i].vendor, devList[i].product );
                p2k_setP2Kconfig( devList[i].vendor, devList[i].product -1);
            }
            return true;
        }
        i++;
    }
    return false;
}


extern "C" { QValueList<p2k_fileInfo> kiop2k_fileList; }

extern "C"
        void onFile( p2k_fileInfo file)
{
    kiop2k_fileList+=(file);
}


P2KWrapper::~P2KWrapper()
{
    kdDebug() << "P2KWrapper::~P2KWrapper()\n";
    kdDebug() << "P2KWrapper::ClosingPhone\n";
    p2k_closePhone();
}


#include "p2kwrapper.moc"


/*!
    \fn P2KWrapper::connect()
 */
bool P2KWrapper::connectPhone()
{
    kdDebug() << " P2KWrapper::connectPhone()\n";
    if (b_connected) return TRUE;
    int err=p2k_findPhone();
    kdDebug() << "err==" << err << endl;
    if(err==P2K_PHONE_NONE)
    {
        kdDebug() << "Phone not found\n";
        return FALSE;
    }
    if(err==P2K_PHONE_AT)
    {
        kdDebug() << "Phone found in AT mode\n";
        if(p2k_setP2Kmode(1000) )
        {
            kdDebug() << "Could not set phone to p2k mode\n";
            return FALSE;
        }
    }
    for( int i=0; i<10 ; i++ ){
        err=p2k_openPhone(100000);
        if( err ) kdDebug() << "Can't connect to the phone (try " << i << ", error number: " << err << ")\n";
        else break;
        sleep (4);
    }
    if (err) return FALSE;
    kdDebug() << "Phone connected in p2k mode\n";
    b_connected=TRUE;
    int i_files=p2k_fileCount();
//     if (i_files != n_files) fetchFileList(FALSE);
    return b_connected;
}

void P2KWrapper::closePhone()
{
    kdDebug() << "P2KWrapper::closePhone()\n";
    if (  p2k_closePhone() )
    {
        kdDebug() << "!!!!!!!!!!!!!!!! Error while closing phone\n";
        return;
    }
    b_connected=FALSE;
    kdDebug() << "**************** Phone disconnected\n";
}

/*!
    \fn P2KWrapper::getRoot()
 */
QStringList P2KWrapper::getRoot()
{
    kdDebug() << "P2KWrapper::getRoot()\n";
    if (!connectPhone() ) return QStringList(QString::null);
    char *buffer=new char[256];
    int err=p2k_FSAC_getVolumes( buffer );
    if( err )
    {
        delete [] buffer;
        closePhone();
        return QStringList(QString::null);
    }
    QString drives( (char*) buffer);
    delete [] buffer;
    kdDebug() << drives << endl;
    if(drives=="FLASH")
    {
        closePhone();
        return QStringList("/");
    }
    drives=drives.replace('/', "");
    closePhone();
    return QStringList::split(" ", drives);
}

void P2KWrapper::fetchFileList(bool close)
{
    kdDebug() << "P2KWrapper::fetchFileList(bool close=" << close << ")\n";
    if (!connectPhone() ) 
    {
        kdDebug() << "***** ERROR: Couldn't connect to the phone\n";
        return;
    }
    files.clear();
    dirs.clear();
    ::kiop2k_fileList.clear();
    p2k_FSAC_fileList( onFile );
    kdDebug() << "************************************* Finished listing files **********************************\n";
    files=::kiop2k_fileList;
    dirs.clear();
    QValueListIterator<p2k_fileInfo> it;
    QString filename;
    for( it=files.begin(); it!=files.end(); ++it)
    {
        filename=QString( (*it).name);
        filename=filename.left( filename.findRev('/') ) ;
        if( dirs.findIndex( filename ) <0 ) dirs.append( filename );
        for(int i=0; i<filename.contains('/'); i++)
            if( dirs.findIndex( filename.left(filename.find('/',i) ) ) <0 ) dirs.append( filename.left(filename.find('/',i) ) );
    }
    n_files=files.count();
    if(close) closePhone();
}




/**
 * @brief Get a file
 * @param name the name of the file.
 * @param buffer a pointer to the buffer, or NULL if you need only to discover the file size.
 * @return File size if successful, 0 otherwise.
 */
int P2KWrapper::getFile(const QString &name, char* buffer)
{
    kdDebug() << "P2KWrapper::getFile(" << name << ",...)\n";
    if( ! connectPhone() ) return 0;
    p2k_FSAC_close();
//     QByteArray retval=0;
    p2k_fileInfo file;
    bool found=FALSE;
    QValueListIterator<p2k_fileInfo> files_it;
    for(files_it=files.begin(); files_it!=files.end(); ++files_it)
    {
        if( name==(*files_it).name )
        {
            file=(*files_it);
            found=TRUE;
            break;
        }
    }
    if(!found) {     closePhone();        return 0; }
    if( ! buffer ) return file.size;
    kdDebug() << "kio_p2k: ******* trying to open " << file.name << endl;
    if( p2k_FSAC_open( file.name, file.attr ) ) {     closePhone();        return 0; }
    kdDebug() << "kio_p2k: Open done, now reading to uchar[" << file.size << "]\n";
/*    if( ! p2k_FSAC_read( (uchar*) buffer, file.size) )
    {
        p2k_FSAC_close();
        return file.size;
    }*/
    int pt=0;
    while( pt<file.size ){
        kdDebug() << "Seek success #" << pt << ":" << p2k_FSAC_seek( pt, P2K_SEEK_BEGIN ) << endl;
        if(file.size-pt >= 0x400 )
            p2k_FSAC_read( (uchar*) &buffer[pt], 0x400 );
        else p2k_FSAC_read( (uchar*) &buffer[pt], file.size % 0x400 );

        pt+=0x400;
    }
//     for (int i=0; i<file.size; i++ )
//     {
//         cout << (int) cbuffer [ i ] << "   ";
//         if(! i%10) cout << endl;
//     }
    p2k_FSAC_close();
    closePhone();
    return file.size;
}

bool P2KWrapper::putFile( const QString &name, char* buffer, int size)
{
    kdDebug() << "P2KWrapper::putFile(" << name << ",.......,......)\n";
    if( ! connectPhone() ) return FALSE;

    kdDebug() << "Writing " << name << " to the phone, size: " << size << "bytes.\n";
    kdDebug() << "Opening connection: " << p2k_FSAC_open( (char*) name.latin1(), '\0' ) << endl;
    int pt=0;
    while ( pt<size )
    {
        kdDebug() << "Seek to trunk #" << pt << ": " << p2k_FSAC_seek( pt, P2K_SEEK_BEGIN ) << endl;
        if(size>=0x400) p2k_FSAC_write( (uchar*) &buffer[pt], 0x400 );
        else p2k_FSAC_write( (uchar*) &buffer[pt], size%0x400 );
        pt+=0x400;
    }
    p2k_FSAC_close();
    closePhone();
    return TRUE;
}

bool P2KWrapper::deleteFile( const QString &name )
{
    kdDebug() << "P2KWrapper::deleteFile(" << name << ")\n";
    if( ! connectPhone() ) return FALSE;
    int result=p2k_FSAC_delete( (char*) name.latin1() );
    kdDebug() << "Deleting " << name << ": " << result << endl;
//     p2k_FSAC_close();
    closePhone();
    return !result;
}
